PathEditScreen {
    layout: grid;
    grid-size: 1 2;
    grid-gutter: 1;
    grid-rows: 10fr 1fr;
}

#path-edit-button-container {
    width: 100%;
    height: 100%;
    align: center middle;
}

TreeNode.selected {
    background: $primary-darken-2;
}